#include <LiquidCrystal.h>
#include <avr/wdt.h>
#include <PinChangeInt.h>
#include <avr/sleep.h>
#include <avr/power.h>
#include <avr/wdt.h>
#include <avr/interrupt.h>
#include <TimerOne.h>
//===================================================================
#define echoPin         A5                          
#define trigPin         10
#define vccSonar        A4
#define vccLcd          A3
#define wakePin         2
#define sleepPin        16
#define NO3             A0
#define NO4             A1

#define lcdRS           8
#define lcdEN           9
#define lcdD4           4
#define lcdD5           5
#define lcdD6           6
#define lcdD7           7

//===================================================================
#ifndef cbi
#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#endif
//===================================================================
#define S                 218
#define TH                214
#define KSI               216
#define degree            178
#define FE                0
#define PS                1
#define OM                2
#define G                 3
#define D                 4
#define L                 5
#define P                 6
#define BATTERY           7
//===================================================================
float celsius;
float farenheit;
float duration;
float distance, correctedSoundSpeed, distanceMetric, distanceUS, soundSpeed;
float voltage;
const long scaleConst = 1156.300 * 1000 ;

boolean sleepFlag = false;             // variable to store a request for sleep
int secCounter = 0;
boolean centimeters = true;
boolean inches = false;
boolean outOfRange = false;

//===================================================================
byte FI[8] = {
	B01110,
	B10101,
	B10101,
	B10101,
	B01110,
	B00100,
	B00100,
	B00000
};
byte PSI[8] = {
	B10101,
	B10101,
	B10101,
	B01110,
	B00100,
	B00100,
	B00100,
	B00000
};
byte OMEGA[8] = {
	B01110,
	B10001,
	B10001,
	B10001,
	B01110,
	B00000,
	B11111,
	B00000
};
byte GAMMA[8] = {
	B11111,
	B10000,
	B10000,
	B10000,
	B10000,
	B10000,
	B10000,
	B00000
};
byte DELTA[8] = {
	B00100,
	B01010,
	B10001,
	B10001,
	B10001,
	B10001,
	B11111,
	B00000
};
byte LAMDA[8] = {
	B00100,
	B01010,
	B10001,
	B10001,
	B10001,
	B10001,
	B10001,
	B00000
};

byte PEE[8] = {
	B11111,
	B10001,
	B10001,
	B10001,
	B10001,
	B10001,
	B10001,
	B00000
};
byte BATT[8] = {
	B01110,
	B10001,
	B10001,
	B11111,
	B11111,
	B11111,
	B11111,
	B00000
};
//===================================================================
LiquidCrystal lcd(8, 9, 4, 5, 6, 7);
//===================================================================
void setup()
{  
  pinMode(NO3, INPUT_PULLUP); 
  pinMode(NO4, INPUT_PULLUP); 
  pinMode(wakePin, INPUT_PULLUP); 
  pinMode(sleepPin, INPUT_PULLUP);  
  pinMode(echoPin, INPUT);  
  pinMode(trigPin, OUTPUT);
  pinMode(vccLcd, OUTPUT); 
  pinMode(vccSonar, OUTPUT); 
  
  PCintPort::detachInterrupt(wakePin);
 
  powerUpLcd();
  powerUpSonar();
  
  lcd.createChar(0, FI);
  lcd.createChar(1, PSI);
  lcd.createChar(2, OMEGA);
  lcd.createChar(3, GAMMA);
  lcd.createChar(4, DELTA);
  lcd.createChar(5, LAMDA);
  lcd.createChar(6, PEE);
  lcd.createChar(7, BATT);
  lcd.clear();
  splashScreen(); 
  
  
}
//===================================================================
void showVital()
{
  secCounter = 0;
  readTemp();
  readVcc();
  lcd.clear(); 
  lcd.print("Vcc= "); printFloat(voltage/1000, 2); lcd.print(" V"); 
  lcd.setCursor(0,1);
  lcd.print("Tc= "); printFloat(celsius, 1); lcd.print(" "); lcd.write(degree); lcd.print("C");
  lcd.setCursor(0,2);
  lcd.print("Tf= "); printFloat(farenheit, 1); lcd.print(" "); lcd.write(degree); lcd.print("F");
  lcd.setCursor(0,3);
  lcd.print("Us= "); printFloat(soundSpeed, 2); lcd.print(" m/s");
  delay(5000);
}  
//===================================================================
void loop()
{   
  distance = readDistance(); 
  readVcc(); 
  readTemp(); 
  
  if(outOfRange == false)
  {
  lcd.setCursor(0,0);
  printFloat(distanceMetric, 2); lcd.print(" EKATO"); lcd.write(S); lcd.print("TA      ");
  lcd.setCursor(0,1);
  printFloat(distanceUS, 2); lcd.print(" INT"); lcd.write(S); lcd.print("E"); lcd.write(S); lcd.print("      ");
  }
  else
  {
    lcd.setCursor(0,0);
    lcd.print("    EKTO"); lcd.write(S); lcd.print(" EYPOY"); lcd.write(S); lcd.print("    ");
    lcd.setCursor(0,1);
    lcd.print("  METPH"); lcd.write(S); lcd.print("H A"); lcd.write(D); lcd.print("YNATH   ");
  }  
  lcd.setCursor(0,2);
  lcd.print("[1] = ON / RESET    "); 
  lcd.setCursor(0,3);
  lcd.print("[2] = OFF   ");
  lcd.write(BATTERY); lcd.print(" "); printFloat(voltage/1000, 2); lcd.print(" V"); 
  
  delay(500);
  secCounter++;
  if((digitalRead(sleepPin)==LOW) || (secCounter >= 40))
  {
    secCounter = 0;
    sleepNow();
  }
  
  if(digitalRead(NO3)==LOW)
  {    
    showVital();
  }  
  
  if(digitalRead(NO4)==LOW)
  {    
    secCounter = 0;
    lcd.clear(); 
    delay(5000);
  }  
}  
//===================================================================
void powerUpLcd()
{  
  digitalWrite(vccLcd, HIGH);
  lcd.begin(20,4);
}  
//===================================================================
void powerUpSonar()
{  
  digitalWrite(vccSonar, HIGH);
} 
//===================================================================
void powerDownLcd()
{ 
  digitalWrite(vccLcd, LOW);
}  
//===================================================================
void powerDownSonar()
{
  digitalWrite(vccSonar, LOW);
}  
//===================================================================
void readButtons()
{
  if(digitalRead(wakePin == LOW))
  {    
    wakeUpNow();
    soft_reset();
  }  
}  
//===================================================================
void printFloat(float value, int places) 
{  
  int digit;
  float tens = 0.1;
  int tenscount = 0;
  int i;
  float tempfloat = value;
  float d = 0.5;
  if (value < 0) d *= -1.0;
  for (i = 0; i < places; i++) d/= 10.0;    
  tempfloat +=  d;
  if (value < 0) tempfloat *= -1.0;
  while ((tens * 10.0) <= tempfloat) 
  {
    tens *= 10.0;
    tenscount += 1;
  }
  if (value < 0) lcd.print('-');
  if (tenscount == 0) lcd.print(0, DEC);
  for (i=0; i< tenscount; i++) 
  {
    digit = (int) (tempfloat/tens);
    lcd.print(digit, DEC);
    tempfloat = tempfloat - ((float)digit * tens);
    tens /= 10.0;
  }
  if (places <= 0) return;
  lcd.print('.');  
  for (i = 0; i < places; i++) 
  {
    tempfloat *= 10.0;
    digit = (int) tempfloat;
    lcd.print(digit,DEC);  
    tempfloat = tempfloat - (float) digit;
  }
}
//===================================================================
float microseconds_to_centimeters(float microseconds) 
{
   return microseconds / 29.387 / 2;
}
//===================================================================
float microseconds_to_inches(float microseconds) 
{
   return microseconds / 74 / 2;
}
//===================================================================
float readDistance()
{
  powerUpSonar();

  digitalWrite(trigPin, LOW);                   // Set the trigger pin to low for 2uS
  delayMicroseconds(2);
  digitalWrite(trigPin, HIGH);                  // Send a 10uS high to trigger ranging
  delayMicroseconds(10);
  digitalWrite(trigPin, LOW);                   // Send pin low again 
  duration = pulseIn(echoPin, HIGH); 
  distanceMetric = (microseconds_to_centimeters(duration))*correctedSoundSpeed;
  distanceUS = (microseconds_to_inches(duration))*correctedSoundSpeed;
  if(distanceMetric == 0) outOfRange = true;
  else outOfRange = false;
}  
//===================================================================
void wakeUpNow() 
{ 
  sleep_disable();
  sei(); 
  UltraPowerDown(true);
  PCintPort::detachInterrupt(wakePin); 
}  
//===================================================================
void sleepNow() 
{ 
  sleepFlag = true; 
  lcd.clear();
  lcd.print("H "); lcd.write(S); lcd.print("Y"); lcd.write(S); lcd.print("KEYH "); lcd.write(TH); lcd.print("A "); lcd.write(S); lcd.print("BH"); lcd.write(S); lcd.print("EI");    
  delay(3000);
  powerDownLcd();
  powerDownSonar();
  byte adcsra = ADCSRA;          
  ADCSRA = 0;                    
  EICRA = _BV(ISC01);            
  EIMSK = _BV(INT0);             
  UltraPowerDown(false);
  PCintPort::attachInterrupt(wakePin, &readButtons, LOW); 
  set_sleep_mode(SLEEP_MODE_PWR_DOWN);
  cli();                         
  sleep_enable();
 
  uint8_t mcucr1 = MCUCR | _BV(BODS) | _BV(BODSE);
  uint8_t mcucr2 = mcucr1 & ~_BV(BODSE);
  MCUCR = mcucr1;
  MCUCR = mcucr2;
  
  sei();                         
  sleep_cpu();                   
  sleep_disable();               
  ADCSRA = adcsra;               
}  
//===================================================================
void UltraPowerDown(boolean onoff)
{
  if(onoff == false)                                                    
  {
    lcd.off();
    powerDownLcd();
    powerDownSonar();
  }
  else
  {
    powerUpLcd();    
    powerUpSonar();
  }  
}
//===================================================================
void splashScreen()
{
  lcd.setCursor(0,0);
  lcd.print("  ");
  lcd.write(S); 
  lcd.print("Y"); 
  lcd.write(S); 
  lcd.print("HMA METPH"); 
  lcd.write(S); 
  lcd.print("H"); 
  lcd.write(S); 
  lcd.print("   ");
  
  lcd.setCursor(0,1);
  lcd.print("     A"); 
  lcd.write(P); 
  lcd.print("O"); 
  lcd.write(S); 
  lcd.print("TA"); 
  lcd.write(S); 
  lcd.print("H"); 
  lcd.write(S); 
  lcd.print("      ");
  
  lcd.setCursor(0,2);
  lcd.print("    ME Y"); 
  lcd.write(P); 
  lcd.print("EPHXOY"); 
  lcd.write(S); 
  lcd.print("    ");
  delay(2500);
  lcd.clear();
  
  lcd.setCursor(0,1);
  lcd.print("     MANOS MAR.     "); 
    
  lcd.setCursor(0,2);
  lcd.print("     2014-2015      ");
  delay(2500);
  lcd.clear();
}  
//===================================================================
void soft_reset()
{
  wdt_enable(WDTO_15MS); 
  for(;;) 
  { 
  
  } 
}  
//===================================================================
void readTemp() 
{
  long result;
  
  ADMUX = _BV(REFS1) | _BV(REFS0) | _BV(MUX3);
  delay(2); 
  ADCSRA |= _BV(ADSC); 
  while (bit_is_set(ADCSRA,ADSC));
  result = ADCL;
  result |= ADCH<<8;
  result = (result - 125) * 1075;
  celsius = result/10000;
  farenheit = celsius*9/5+32;
  correctedSoundSpeed =  331.5*(sqrt(1+(celsius/273.15)))/340;
  soundSpeed = 331.5*(sqrt(1+(celsius/273.15))); 
}
//===================================================================
void readVcc() 
{
  long result;
  
  ADMUX = _BV(REFS0) | _BV(MUX3) | _BV(MUX2) | _BV(MUX1);
  delay(2); 
  ADCSRA |= _BV(ADSC); 
  while (bit_is_set(ADCSRA,ADSC));
  uint8_t low  = ADCL; 
  uint8_t high = ADCH; 
  result = (high<<8) | low;
  result = scaleConst / result;
  voltage = result;
}
//===================================================================
//===================================================================
//===================================================================
//===================================================================
//===================================================================
//===================================================================
